@@ -1,6 +1,6 @@
 use core::{cell::RefCell, str::FromStr};
 
-use kwap_msg::{EnumerateOptNumbers, TryIntoBytes};
+use kwap_msg::TryIntoBytes;
 use no_std_net::{Ipv4Addr, SocketAddrV4};
 use tinyvec::ArrayVec;
 
@@ -15,6 +15,7 @@ use self::event::listeners::log;
 use crate::{config::{self, Config},
             req::Req,
             resp::Resp,
+            result_ext::ResultExt,
             socket::Socket};
 
 /// A CoAP request/response runtime that drives client- and server-side behavior.
@@ -33,6 +34,19 @@ pub struct Core<Sock: Socket, Cfg: Config> {
   resps: RefCell<ArrayVec<[Option<Resp<Cfg>>; 64]>>,
 }
 
+/// An error encounterable while sending a message
+#[derive(Debug, Clone)]
+pub enum SendError<Cfg: Config, Sock: Socket> {
+  /// Some socket operation (e.g. connecting to host) failed
+  SockError(Sock::Error),
+  /// Serializing a message to bytes failed
+  ToBytes(<config::Message<Cfg> as TryIntoBytes>::Error),
+  /// Uri-Host in request was not a utf8 string
+  HostInvalidUtf8(core::str::Utf8Error),
+  /// Uri-Host in request was not a valid IPv4 address (TODO)
+  HostInvalidIpAddress,
+}
+
 impl<Sock: Socket, Cfg: Config> Core<Sock, Cfg> {
   /// Creates a new Core with the default runtime behavior
   pub fn new(sock: Sock) -> Self {
@@ -152,32 +166,36 @@ impl<Sock: Socket, Cfg: Config> Core<Sock, Cfg> {
   }
 
   /// Check the stored socket for a new datagram, and fire a RecvDgram event
-  fn poll_sock(&mut self) {
-    let mut buf = [0u8; 1152];
-    let recvd = self.sock.recv(&mut buf);
-    match recvd {
-      | Ok(n) => {
-        let ev = Event::RecvDgram(Some(buf[0..n].iter().copied().collect()));
-        self.fire(ev);
-      },
-      | _ => {},
-    }
-  }
-
   /// Poll for a response to a sent request
   ///
   /// # Example
   /// See `./examples/client.rs`
   pub fn poll_resp(&mut self, req_id: kwap_msg::Id) -> nb::Result<Resp<Cfg>, Sock::Error> {
-    self.poll_sock();
-    let mut resps = self.resps.borrow_mut();
-    let id_matches = |o: &Option<Resp<Cfg>>| o.as_ref().unwrap().msg.id == req_id;
-    let resp = resps.iter_mut().find_map(|rep| match rep {
-                                 | mut o @ Some(_) if id_matches(&o) => Option::take(&mut o),
-                                 | _ => None,
-                               });
+    // check if there's a dgram in the socket,
+    // and move it through the event pipeline.
+    //
+    // this will store the response (if there is one) before we continue.
+    let resp_matches = |o: &Option<Resp<Cfg>>| o.as_ref().unwrap().msg.id == req_id;
 
-    resp.ok_or(nb::Error::WouldBlock)
+    self.sock
+        .poll()
+        .map(|polled| {
+          if let Some(dgram) = polled {
+            self.fire(Event::RecvDgram(Some(dgram)));
+          }
+          ()
+        })
+        .map_err(nb::Error::Other)
+        .bind(|_| {
+          self.resps
+              .borrow_mut()
+              .iter_mut()
+              .find_map(|rep| match rep {
+                | mut o @ Some(_) if resp_matches(&o) => Option::take(&mut o),
+                | _ => None,
+              })
+              .ok_or(nb::Error::WouldBlock)
+        })
   }
 
   /// Send a message
@@ -191,28 +209,25 @@ impl<Sock: Socket, Cfg: Config> Core<Sock, Cfg> {
   /// let mut core = Core::<_, Alloc>::new(sock);
   /// core.send_req(Req::<Alloc>::get("1.1.1.1", 5683, "/hello"));
   /// ```
-  pub fn send_req(&mut self, req: Req<Cfg>) -> Result<(), ()> {
-    let msg = config::Message::<Cfg>::from(req);
-    let (_, host) = msg.opts
-                       .iter()
-                       .enumerate_option_numbers()
-                       .find(|(n, _)| n.0 == 3)
-                       .unwrap();
-    let (_, port) = msg.opts
-                       .iter()
-                       .enumerate_option_numbers()
-                       .find(|(n, _)| n.0 == 7)
-                       .unwrap();
-    let port_bytes = port.value.0.iter().copied().collect::<ArrayVec<[u8; 2]>>();
+  pub fn send_req(&mut self, req: Req<Cfg>) -> Result<(), SendError<Cfg, Sock>> {
+    let port = req.get_option(7).expect("Uri-Port must be present");
+    let port_bytes = port.value.0.iter().take(2).copied().collect::<ArrayVec<[u8; 2]>>();
     let port = u16::from_be_bytes(port_bytes.into_inner());
-    let host_str: &str = core::str::from_utf8(&host.value.0).unwrap();
-    self.sock
-        .connect(SocketAddrV4::new(Ipv4Addr::from_str(host_str).unwrap(), port))
-        .map_err(|_| ())?;
-    self.sock
-        .send(&msg.try_into_bytes::<ArrayVec<[u8; 1152]>>().map_err(|_| ())?)
-        .map_err(|_| ())?;
-    Ok(())
+
+    let host = req.get_option(3).expect("Uri-Host must be present");
+    core::str::from_utf8(&host.value.0).map_err(SendError::HostInvalidUtf8)
+                                       .bind(|host| {
+                                         Ipv4Addr::from_str(host).map_err(|_| SendError::HostInvalidIpAddress)
+                                       })
+                                       .map(|ip| SocketAddrV4::new(ip, port))
+                                       .try_perform(|addr| self.sock.connect(addr).map_err(SendError::SockError))
+                                       .bind(|_| {
+                                         req.try_into_bytes::<ArrayVec<[u8; 1152]>>().map_err(SendError::ToBytes)
+                                       })
+                                       .try_perform(|bytes| {
+                                         nb::block!(self.sock.send(bytes)).map_err(SendError::SockError)
+                                       })
+                                       .map(|_| ())
   }
 }
 
