@@ -23,7 +23,7 @@
 //! - Because UDP is a "connectionless" protocol, it offers no guarantee of "conversation" between traditional client and server roles. All the UDP transport layer gives you is a method to listen for messages thrown at you, and to throw messages at someone. Owing to this, CoAP machines are expected to perform both client and server roles (or more accurately, _sender_ and _receiver_ roles)
 //! - While _classes_ of status codes are the same (Success 2xx -> 2.xx, Client error 4xx -> 4.xx, Server error 5xx -> 5.xx), the semantics of the individual response codes differ.
 
-#![doc(html_root_url = "https://docs.rs/kwap/0.1.6")]
+#![doc(html_root_url = "https://docs.rs/kwap/0.1.8")]
 #![cfg_attr(all(not(test), feature = "no_std"), no_std)]
 #![cfg_attr(not(test),
             deny(missing_debug_implementations,
@@ -36,6 +36,8 @@
 #[cfg(feature = "alloc")]
 extern crate alloc as std_alloc;
 
+pub(crate) mod result_ext;
+
 /// CoAP response messages
 pub mod resp;
 
@@ -153,24 +155,34 @@ pub(crate) mod test {
   impl Socket for TubeSock {
     type Error = Option<()>;
 
-    fn connect<A: ToSocketAddrs>(&mut self, a: A) -> nb::Result<(), Self::Error> {
+    fn connect<A: ToSocketAddrs>(&mut self, a: A) -> Result<(), Self::Error> {
       self.0 = a.to_socket_addrs().unwrap().next();
       Ok(())
     }
 
-    fn recv(&mut self, buf: &mut [u8]) -> nb::Result<usize, Self::Error> {
+    fn recv(&self, buf: &mut [u8]) -> nb::Result<usize, Self::Error> {
       if self.1.is_empty() {
         println!("TubSock recv invoked without sending first");
         return Err(nb::Error::WouldBlock);
       }
 
       let n = self.1.len();
-      self.1.drain(..).enumerate().for_each(|(ix, el)| buf[ix] = el);
+      let vec = &self.1 as *const _ as *mut Vec<u8>;
+      unsafe {
+        vec.as_mut()
+           .unwrap()
+           .drain(..)
+           .enumerate()
+           .for_each(|(ix, el)| buf[ix] = el);
+      }
       Ok(n)
     }
 
-    fn send(&mut self, buf: &[u8]) -> nb::Result<(), Self::Error> {
-      self.1 = buf.iter().copied().collect();
+    fn send(&self, buf: &[u8]) -> nb::Result<(), Self::Error> {
+      let vec = &self.1 as *const _ as *mut Vec<u8>;
+      unsafe {
+        *vec = buf.iter().copied().collect();
+      }
       Ok(())
     }
   }
